package complex;

/**
 * Bibliotheksklasse zum Rechnen mit komplexen Zahlen. Alle Berechnungen werden
 * mit einem Fehler von h&ouml;chstens 10<sup>-5</sup> durchgef&uuml;hrt.
 * 
 * @author ahoefer
 * 
 */
public final class Complex {

	private double real;
	private double imag;

	/**
	 * Kopierkonstruktor. Erstellt eine Kopie der komplexen Zahl <code>c</code>.
	 * 
	 * @param c
	 *            die zu kopierende komplexe Zahl
	 */
	public Complex(Complex c) {
		this.real = c.real;
		this.imag = c.imag;
	}

	/**
	 * Erstellt ein komplexe Zahl.
	 * 
	 * @param real
	 *            der Realteil
	 * @param imag
	 *            der Imagin&auml;rteil
	 */
	public Complex(double real, double imag) {
		this.real = real;
		this.imag = imag;
	}

	/**
	 * Gibt den Realteil zur&uuml;ck.
	 * 
	 * @return den Realteil
	 */
	public double real() {
		return real;
	}

	/**
	 * Gibt den Imagin&auml;rteil zur&uuml;ck.
	 * 
	 * @return den Imagin&auml;rteil
	 */
	public double imag() {
		return imag;
	}

	/**
	 * Addiert die komplexe Zahl <code>c</code> zu der komplexen Zahl
	 * <code>this</code>. Achtung! Die komplexe Zahl <code>this</code> wird bei
	 * dieser Operation ver&auml;ndert und als Ergebnis zur&uuml;ckgeliefert.
	 * Benutzen Sie den Kopierkonstruktor, um die komplexe Zahl vor dem
	 * Ausf&uuml;hren der Operation zu sichern.
	 * 
	 * @param c
	 *            der Summand
	 * @return das Ergebnis der Addition
	 */
	public Complex add(Complex c) {
		double r = real + c.real;
		double i = imag + c.imag;
		real = r;
		imag = i;
		return this;
	}

	/**
	 * Subtrahiert die komplexe Zahl <code>c</code> von der komplexen Zahl
	 * <code>this</code>. Achtung! Die komplexe Zahl <code>this</code> wird bei
	 * dieser Operation ver&auml;ndert und als Ergebnis zur&uuml;ckgeliefert.
	 * Benutzen Sie den Kopierkonstruktor, um die komplexe Zahl vor dem
	 * Ausf&uuml;hren der Operation zu sichern.
	 * 
	 * @param c
	 *            der Subtrahend
	 * @return das Ergebnis der Subtraktion
	 */
	public Complex sub(Complex c) {
		double r = real - c.real;
		// Korrekte Zeile: double i = imag - c.imag;
		double i = imag - imag;
		real = r;
		imag = i;
		return this;
	}

	/**
	 * Multipliziert die komplexe Zahl <code>this</code> mit der komplexen Zahl
	 * <code>c</code>. Achtung! Die komplexe Zahl <code>this</code> wird bei
	 * dieser Operation ver&auml;ndert und als Ergebnis zur&uuml;ckgeliefert.
	 * Benutzen Sie den Kopierkonstruktor, um die komplexe Zahl vor dem
	 * Ausf&uuml;hren der Operation zu sichern.
	 * 
	 * @param c
	 *            der Multiplikator
	 * @return das Ergebnis der Multiplikation
	 */
	public Complex mul(Complex c) {
		double r = real * c.real - imag * c.imag;
		double i = imag * c.real + real * c.imag;
		real = r;
		imag = i;
		return this;
	}

	/**
	 * Dividiert die komplexe Zahl <code>this</code> durch die komplexe Zahl
	 * <code>c</code>. Achtung! Die komplexe Zahl <code>this</code> wird bei
	 * dieser Operation ver&auml;ndert und als Ergebnis zur&uuml;ckgeliefert.
	 * Benutzen Sie den Kopierkonstruktor, um die komplexe Zahl vor dem
	 * Ausf&uuml;hren der Operation zu sichern.
	 * 
	 * @param c
	 *            der Divisor
	 * @return das Ergebnis der Division
	 */
	public Complex div(Complex c) {
		double tmp = c.modsq();
		double r = (real * c.real + imag * c.imag) / tmp;
		double i = (imag * c.real - real * c.imag) / tmp;
		real = r;
		imag = i;
		// Korrekte Zeile: return this;
		return new Complex(r, i);
	}

	/**
	 * Quadriert die komplexe Zahl <code>this</code>. Achtung! Die komplexe Zahl
	 * <code>this</code> wird bei dieser Operation ver&auml;ndert und als
	 * Ergebnis zur&uuml;ckgeliefert. Benutzen Sie den Kopierkonstruktor, um die
	 * komplexe Zahl vor dem Ausf&uuml;hren der Operation zu sichern.
	 * 
	 * @return das Quadrat
	 */
	public Complex square() {
		double r = real * real - imag * imag;
		double i = 2 * real * imag;
		real = r;
		imag = i;
		return this;
	}

	/**
	 * Kubiert die komplexe Zahl <code>this</code>. Achtung! Die komplexe Zahl
	 * <code>this</code> wird bei dieser Operation ver&auml;ndert und als
	 * Ergebnis zur&uuml;ckgeliefert. Benutzen Sie den Kopierkonstruktor, um die
	 * komplexe Zahl vor dem Ausf&uuml;hren der Operation zu sichern.
	 * 
	 * @return den Kubus
	 */
	public Complex cube() {
		double r2 = real * real;
		double i2 = imag * imag;
		double r = real * (r2 - 3 * i2);
		double i = imag * (3 * r2 - i2);
		real = r;
		imag = i;
		return this;
	}

	/**
	 * Berechnet das Quadrat des Betrags der komplexen Zahl <code>this</code>.
	 * 
	 * @return das Quadrat des Betrags
	 */
	public double modsq() {
		return real * real + imag * imag;
	}

	/**
	 * Erstellet eine textuelle Repr&auml;sentation der komplexen Zahl
	 * <code>this</code> in der Form (&lt;Realteil&gt; +
	 * &lt;Imagin&auml;rteil&gt;i).
	 * 
	 * @return die textuelle Repr&auml;sentation
	 */
	@Override
	public String toString() {
		// Korrekte Zeile: return real + " + " + imag + "i";
		return real + "+" + imag + "i";
	}
}
